@@ -149,7 +149,7 @@ void ant_c::draw(void) {
 
   if (gr.getAnt(animation, animationImage))
   {
-    vid.blit(gr.getAnt(animation, animationImage), (blockX-2)*gr.blockX(), (blockY)*gr.blockY()+screenBlock+gr.antDisplace());
+    vid.blit(gr.getAnt(animation, animationImage), blockX*gr.blockX()-45, (blockY)*gr.blockY()+screenBlock+gr.antDisplace()+3);
   }
 
   /* what comes now, is to put the ladders back in front of the ant,
@@ -166,16 +166,16 @@ void ant_c::draw(void) {
   }
   else
   {
-    if (level.getFg(blockX, blockY) == levelData_c::FgElementPlatformLadderDown ||
-        level.getFg(blockX, blockY) == levelData_c::FgElementLadder)
+    if ((level.getFg(blockX, blockY) == levelData_c::FgElementPlatformLadderDown) ||
+        (level.getFg(blockX, blockY) == levelData_c::FgElementLadder))
     {
       vid.blitBlock(gr.getFgTile(levelData_c::FgElementLadder), (blockX)*gr.blockX(), (blockY)*gr.blockY());
     }
   }
 
-  if (blockY > 0 &&
-      level.getFg(blockX, blockY-1) == levelData_c::FgElementPlatformLadderDown ||
-      level.getFg(blockX, blockY-1) == levelData_c::FgElementLadder)
+  if ((blockY > 0) &&
+      ((level.getFg(blockX, blockY-1) == levelData_c::FgElementPlatformLadderDown) ||
+       (level.getFg(blockX, blockY-1) == levelData_c::FgElementLadder)))
   {
     vid.blitBlock(gr.getFgTile(levelData_c::FgElementLadder), (blockX)*gr.blockX(), (blockY-1)*gr.blockY());
   }
@@ -188,21 +188,21 @@ void ant_c::draw(void) {
     }
     else
     {
-      if (level.getFg(blockX-1, blockY) == levelData_c::FgElementPlatformLadderDown ||
-          level.getFg(blockX-1, blockY) == levelData_c::FgElementLadder)
+      if ((level.getFg(blockX-1, blockY) == levelData_c::FgElementPlatformLadderDown) ||
+          (level.getFg(blockX-1, blockY) == levelData_c::FgElementLadder))
       {
         vid.blitBlock(gr.getFgTile(levelData_c::FgElementLadder), (blockX-1)*gr.blockX(), (blockY)*gr.blockY());
       }
     }
-    if (blockY > 0 &&
-        level.getFg(blockX-1, blockY-1) == levelData_c::FgElementPlatformLadderDown ||
-        level.getFg(blockX-1, blockY-1) == levelData_c::FgElementLadder)
+    if ((blockY > 0) &&
+        ((level.getFg(blockX-1, blockY-1) == levelData_c::FgElementPlatformLadderDown) ||
+         (level.getFg(blockX-1, blockY-1) == levelData_c::FgElementLadder)))
     {
       vid.blitBlock(gr.getFgTile(levelData_c::FgElementLadder), (blockX-1)*gr.blockX(), (blockY-1)*gr.blockY());
     }
   }
 
-  if (blockX < 19 && vid.isDirty(blockX+1, blockY))
+  if ((blockX < 19) && vid.isDirty(blockX+1, blockY))
   {
     if (level.getFg(blockX+1, blockY) == levelData_c::FgElementPlatformLadderUp)
     {
@@ -210,15 +210,15 @@ void ant_c::draw(void) {
     }
     else
     {
-      if (level.getFg(blockX+1, blockY) == levelData_c::FgElementPlatformLadderDown ||
-          level.getFg(blockX+1, blockY) == levelData_c::FgElementLadder)
+      if ((level.getFg(blockX+1, blockY) == levelData_c::FgElementPlatformLadderDown) ||
+          (level.getFg(blockX+1, blockY) == levelData_c::FgElementLadder))
       {
         vid.blitBlock(gr.getFgTile(levelData_c::FgElementLadder), (blockX+1)*gr.blockX(), (blockY)*gr.blockY());
       }
     }
-    if (blockY > 0 &&
-        level.getFg(blockX+1, blockY-1) == levelData_c::FgElementPlatformLadderDown ||
-        level.getFg(blockX+1, blockY-1) == levelData_c::FgElementLadder)
+    if ((blockY > 0) &&
+        ((level.getFg(blockX+1, blockY-1) == levelData_c::FgElementPlatformLadderDown) ||
+         (level.getFg(blockX+1, blockY-1) == levelData_c::FgElementLadder)))
     {
       vid.blitBlock(gr.getFgTile(levelData_c::FgElementLadder), (blockX+1)*gr.blockX(), (blockY-1)*gr.blockY());
     }
@@ -354,6 +354,13 @@ unsigned int ant_c::callStateFunction(unsigned int state) {
 // when the animation is finished the function returns true
 bool ant_c::animateAnt(unsigned int delay) {
 
+  // to conform the the original, we should use
+  // animationTimer > 1 here, so the original
+  // game is a tiny bit faster for some animations
+  //
+  // but doing that would destroy a huge set of our
+  // recordings, so we don't and live with the tiny
+  // difference
   if (animationTimer > 0) {
     animationTimer--;
     return false;
@@ -682,7 +689,7 @@ unsigned int ant_c::SFLeaveLadderRight(void) {
     return animation;
 
   animation = AntAnimCarryRight;
-  return AntAnimNothing;
+  return animation;
 
 }
 
@@ -692,7 +699,7 @@ unsigned int ant_c::SFLeaveLadderLeft(void) {
     return animation;
 
   animation = AntAnimCarryLeft;
-  return AntAnimNothing;
+  return animation;
 }
 
 unsigned int ant_c::SFEnterLadder(void) {
@@ -1107,17 +1114,17 @@ unsigned int ant_c::checkForNoKeyActions(void) {
         return ReturnAntState;
       }
     }
-    if (level.getFg(blockX, blockY-1) == 4 ||
-        level.getFg(blockX, blockY-1) == 5 &&
-        inactiveTimer > 0x0A0)
+    if ((level.getFg(blockX, blockY-1) == 4 ||
+         level.getFg(blockX, blockY-1) == 5) &&
+         inactiveTimer > 0x0A0)
     {
 
       animation = ReturnAntState = AntAnimLadder1;
       return ReturnAntState;
     }
-    if (level.getFg(blockX, blockY+1) == 5 ||
-        level.getFg(blockX, blockY+1) == 6 ||
-        level.getFg(blockX, blockY+1) == 4 &&
+    if ((level.getFg(blockX, blockY+1) == 5 ||
+         level.getFg(blockX, blockY+1) == 6 ||
+         level.getFg(blockX, blockY+1) == 4) &&
         level.getFg(blockX, blockY) == 5)
     {
       animation = ReturnAntState = AntAnimLadder3;
@@ -1177,14 +1184,14 @@ unsigned int ant_c::checkForNoKeyActions(void) {
   }
 
   // tapping and yawning when nothing else is going on
-  if (((inactiveTimer & 0x20) == 0x20) && ((inactiveTimer & 0x1F) == 5))
+  if (((inactiveTimer & 0x20) == 0x20) && ((inactiveTimer & 0x1F) == 0))
   {
     soundSystem_c::instance()->startSound(soundSystem_c::SE_NU_WHAT);
     animation = AntAnimTapping;
     return AntAnimTapping;
   }
 
-  if (((inactiveTimer & 0x20) == 0x20) && ((inactiveTimer & 0x1F) <= 5))
+  if (((inactiveTimer & 0x20) == 0x20) && ((inactiveTimer & 0x1F) < 5))
   {
     animation = AntAnimTapping;
     return AntAnimTapping;
@@ -1214,15 +1221,17 @@ static void SaveLevelState(void) {
 
 bool ant_c::CanPlaceDomino(int x, int y, int ofs) {
 
+  if (ofs < -1 || ofs > 1) return false;
+
   x += ofs;
 
   if (x < 0 && ofs == -1) return false;
   if (x > 19 && ofs == 1) return false;
 
-  // if the target position is not empty, we can not put the domino there
+  // if the target position is not empty, we can't put the domino there
   if (level.getDominoType(x, y) != levelData_c::DominoTypeEmpty) return false;
 
-  // we can not place dominos at the edge, if we are too close to them
+  // we can't place dominos at the edge, if we are too close to them
   // because we need to step back to do that
   if (level.getFg(x, y) == levelData_c::FgElementPlatformStep1)
   {
@@ -1288,8 +1297,8 @@ unsigned int ant_c::SFNextAction(void) {
   unsigned int returnState;
 
   // is true, when the ant is on a ladder
-  bool onLadder = animation >= AntAnimLadder1 && animation <= AntAnimLadder4 ||
-                  animation >= AntAnimCarryLadder1 && animation <= AntAnimCarryLadder4;
+  bool onLadder = (animation >= AntAnimLadder1 && animation <= AntAnimLadder4) ||
+                  (animation >= AntAnimCarryLadder1 && animation <= AntAnimCarryLadder4);
 
   // when we hav no ground below us and are not on a ladder we need
   // to fall down
@@ -1313,10 +1322,10 @@ unsigned int ant_c::SFNextAction(void) {
   if (animation == AntAnimPushLeft &&
       animationImage == 0)
   {
-    if (level.getDominoType(blockX, blockY) != 0 &&
-        level.getDominoState(blockX, blockY) < 8 ||
-        level.getDominoType(blockX-1, blockY) != 0 &&
-        level.getDominoState(blockX-1, blockY) > 8)
+    if ((level.getDominoType(blockX, blockY) != 0 &&
+         level.getDominoState(blockX, blockY) < 8) ||
+        (level.getDominoType(blockX-1, blockY) != 0 &&
+         level.getDominoState(blockX-1, blockY) > 8))
     {
       animation = returnState = AntAnimDominoDying;
       return returnState;
@@ -1325,10 +1334,10 @@ unsigned int ant_c::SFNextAction(void) {
   if (animation == AntAnimPushRight &&
       animationImage == 0)
   {
-    if (level.getDominoType(blockX, blockY) != 0 &&
-        level.getDominoState(blockX, blockY) > 8 ||
-        level.getDominoType(blockX+1, blockY) != 0 &&
-        level.getDominoState(blockX+1, blockY) < 8)
+    if ((level.getDominoType(blockX, blockY) != 0 &&
+         level.getDominoState(blockX, blockY) > 8) ||
+        (level.getDominoType(blockX+1, blockY) != 0 &&
+         level.getDominoState(blockX+1, blockY) < 8))
     {
       if (blockX < 19)
       {
@@ -1412,9 +1421,6 @@ unsigned int ant_c::SFNextAction(void) {
           direction = -1;
         }
       }
-      else if (numPushsLeft == 0)
-      {
-      }
       else if (!PushableDomino(blockX+1, blockY, -1))
       {
       }
@@ -1526,9 +1532,6 @@ unsigned int ant_c::SFNextAction(void) {
           direction = 1;
         }
       }
-      else if (numPushsLeft == 0)
-      {
-      }
       else if (!PushableDomino(blockX-1, blockY, 1))
       {
       }
@@ -1839,9 +1842,9 @@ unsigned int ant_c::SFNextAction(void) {
         animation = returnState = AntAnimLadder3;
         direction = 20;
       }
-    } else if (level.getFg(blockX, blockY+1) == 6 ||
-        level.getFg(blockX, blockY+1) == 4 &&
-        level.getFg(blockX, blockY) == 5)
+    } else if ((level.getFg(blockX, blockY+1) == 6 ||
+                level.getFg(blockX, blockY+1) == 4) &&
+               level.getFg(blockX, blockY) == 5)
     {
       animation = returnState = AntAnimLadder3;
       direction = 20;
