@@ -15,7 +15,7 @@
 void levelPlayer_c::load(const textsections_c & sections) {
 
   openDoorEntry = openDoorExit = false;
-  triggerFalln = false;
+  resetTriggerFalln();
   finishCheckDone = false;
 
   levelDisplay_c::load(sections);
@@ -257,12 +257,12 @@ bool levelPlayer_c::pushDomino(int x, int y, int dir) {
 void levelPlayer_c::DTA_9(int x, int y) {
   DTA_1(x, y);
 
-  triggerFalln = true;
+  setTriggerFalln();
 }
 void levelPlayer_c::DTA_N(int x, int y) {
   DTA_K(x, y);
 
-  triggerFalln = true;
+  setTriggerFalln();
 }
 
 // this is for the stopper, splitter and exploder dominos, when they
@@ -288,6 +288,8 @@ void levelPlayer_c::DTA_G(int x, int y) {
 void levelPlayer_c::DTA_B(int x, int y) {
   DTA_E(x, y);
 
+  markDirty(x-1, y);
+  markDirty(x-1, y-1);
   markDirty(x+1, y);
   markDirty(x+1, y-1);
   markDirty(x, y-1);
@@ -298,13 +300,20 @@ void levelPlayer_c::DTA_B(int x, int y) {
 // function and mark some more blocks because we split left and
 // right and the normal function will only mark one side
 void levelPlayer_c::DTA_D(int x, int y) {
+
   markDirty(x+1, y-1);
   markDirty(x+1, y);
 
   DTA_4(x, y);
 
+  // mark the splitting stone, that now vanises as dirty
+  // the lower 2 lines are for splitting dust clouds
   if (getDominoState(x, y) == 5)
+  {
     markDirty(x, y-2);
+    markDirty(x-1, y-2);
+    markDirty(x+1, y-2);
+  }
 }
 
 // the final vansher state, remove the vanisher
@@ -334,19 +343,20 @@ void levelPlayer_c::DTA_2(int x, int y) {
     return;
   }
 
-  // if there is a domino left of us, we can not fall down further
+  // if there is a domino left of us, we can't fall down further
   if (getDominoType(x-1, y) != 0)
     return;
 
   // if here is a domino 2 to the left of us and that domino has falln down
-  // far enough to the right, we can not fall farther
+  // far enough to the right, we can't fall farther
   if (x >= 2 && getDominoType(x-2, y) != 0 && getDominoState(x-2, y) > 13)
     return;
 
   // if the domino is a splitter, we need to check, if it is in one of the
   // states where the left halve is far enough down
   // if this is not the case or it is no splitter fall further
-  if (getDominoType(x-2, y) != DominoTypeSplitter ||
+  if (x < 2 ||
+      getDominoType(x-2, y) != DominoTypeSplitter ||
       getDominoState(x-2, y) != 1 &&
       getDominoState(x-2, y) != 10 &&
       getDominoState(x-2, y) != 12 &&
@@ -384,7 +394,8 @@ void levelPlayer_c::DTA_J(int x, int y) {
   // if the domino is a splitter, we need to check, if it is in one of the
   // states where the right halve is far enough down
   // if this is not the case or it is no splitter fall further
-  if (getDominoType(x+2, y) != DominoTypeSplitter ||
+  if (x > 17 ||
+      getDominoType(x+2, y) != DominoTypeSplitter ||
       getDominoState(x+2, y) != 1 &&
       getDominoState(x+2, y) != 9 &&
       getDominoState(x+2, y) != 11 &&
@@ -699,11 +710,21 @@ void levelPlayer_c::DTA_E(int x, int y) {
     // the domino into the next block below
     if (y < 12)
     {
-      setDominoType(x, y+1, getDominoType(x, y));
-      setDominoState(x, y+1, getDominoState(x, y));
-      setDominoDir(x, y+1, getDominoDir(x, y));
-      setDominoYOffset(x, y+1, 0);
-      setDominoExtra(x, y+1, 0x70);
+      if (getDominoType(x, y+1) == DominoTypeEmpty)
+      {
+        setDominoType(x, y+1, getDominoType(x, y));
+        setDominoState(x, y+1, getDominoState(x, y));
+        setDominoDir(x, y+1, getDominoDir(x, y));
+        setDominoYOffset(x, y+1, 0);
+        setDominoExtra(x, y+1, 0x70);
+      }
+      else
+      {
+        DominoCrash(x, y+1, getDominoType(x, y), getDominoExtra(x, y));
+        setDominoYOffset(x, y+1, 0);
+        markDirty(x, y);
+        markDirty(x, y+1);
+      }
     }
 
     // remove the old domino
@@ -720,11 +741,35 @@ void levelPlayer_c::DTA_E(int x, int y) {
   if (isTherePlatform(x, y))
   {
     // we still crash if there is a domino below us
-    if (getDominoType(x, y+1) != DominoTypeEmpty)
+    if (getDominoType(x, y+1) != DominoTypeEmpty &&
+        getDominoType(x, y+1) != DominoTypeAscender
+        )
     {
       DominoCrash(x, y+1, getDominoType(x, y), getDominoExtra(x, y));
     }
 
+    if (getDominoType(x, y+1) == DominoTypeAscender)
+    {
+      if (getDominoState(x, y+1) == 1)
+      {
+        setDominoState(x-1, y+2, 14);
+        setDominoType(x-1, y+2, DominoTypeAscender);
+        setDominoDir(x-1, y+2, -1);
+        setDominoExtra(x-1, y+2, 0x00);
+        setDominoYOffset(x-1, y+2, getDominoYOffset(x, y+1)-16);
+        removeDomino(x, y+1);
+      }
+      if (getDominoState(x, y+1) == 15)
+      {
+        setDominoState(x+1, y+2, 2);
+        setDominoType(x+1, y+2, DominoTypeAscender);
+        setDominoDir(x+1, y+2, 1);
+        setDominoExtra(x+1, y+2, 0x00);
+        setDominoYOffset(x+1, y+2, getDominoYOffset(x, y+1)-16);
+        removeDomino(x, y+1);
+      }
+    }
+
     // no more falling
     setDominoExtra(x, y, 0);
 
@@ -743,7 +788,7 @@ void levelPlayer_c::DTA_E(int x, int y) {
 
   // we can continue, if there is either no domino or no more
   // level below us
-  if (y >= 12 || getDominoType(x, y+1) == DominoTypeEmpty)
+  if (y >= 12 || getDominoType(x, y+1) == DominoTypeEmpty || getDominoYOffset(x, y+1) != 0)
   {
     setDominoYOffset(x, y, getDominoYOffset(x, y)+4);
     markDirty(x, y);
@@ -1143,21 +1188,45 @@ void levelPlayer_c::DTA_A(int x, int y) {
   {
     if (a == 1)
     {
-      setDominoExtra(x-1, y, 0x60);
-      setDominoType(x-1, y, DominoTypeAscender);
-      setDominoState(x-1, y, 14);
-      setDominoDir(x-1, y, -1);
-      setDominoYOffset(x-1, y, getDominoYOffset(x, y)-2);
+      if (getDominoType(x-1, y) == DominoTypeEmpty)
+      {
+        setDominoExtra(x-1, y, 0x60);
+        setDominoType(x-1, y, DominoTypeAscender);
+        setDominoState(x-1, y, 14);
+        setDominoDir(x-1, y, -1);
+        setDominoYOffset(x-1, y, getDominoYOffset(x, y)-2);
+      }
+      else
+      {
+        DominoCrash(x-1, y, getDominoType(x, y), getDominoExtra(x, y));
+
+        markDirty(x-2, y-2);
+        markDirty(x-1, y-2);
+        markDirty(x-2, y-1);
+        markDirty(x-1, y-1);
+      }
     }
     else
     {
       if (y > 0)
       {
-        setDominoExtra(x-1, y-1, 0x60);
-        setDominoType(x-1, y-1, DominoTypeAscender);
-        setDominoState(x-1, y-1, 14);
-        setDominoDir(x-1, y-1, -1);
-        setDominoYOffset(x-1, y-1, getDominoYOffset(x, y)+14);
+        if (getDominoType(x-1, y-1) == DominoTypeEmpty)
+        {
+          setDominoExtra(x-1, y-1, 0x60);
+          setDominoType(x-1, y-1, DominoTypeAscender);
+          setDominoState(x-1, y-1, 14);
+          setDominoDir(x-1, y-1, -1);
+          setDominoYOffset(x-1, y-1, getDominoYOffset(x, y)+14);
+        }
+        else
+        {
+          DominoCrash(x-1, y-1, getDominoType(x, y), getDominoExtra(x, y));
+
+          markDirty(x-2, y-3);
+          markDirty(x-1, y-3);
+          markDirty(x-2, y-2);
+          markDirty(x-1, y-2);
+        }
       }
     }
 
@@ -1175,27 +1244,51 @@ void levelPlayer_c::DTA_A(int x, int y) {
   {
     removeDomino(x, y);
 
-    setDominoExtra(x, y+1-a, 0x60);
-    setDominoType(x, y+1-a, DominoTypeAscender);
-    setDominoState(x, y+1-a, 8);
-    setDominoDir(x, y+1-a, -1);
-    setDominoYOffset(x, y+1-a, 0);
+    if (getDominoType(x, y+1-a) == DominoTypeEmpty)
+    {
+      setDominoExtra(x, y+1-a, 0x60);
+      setDominoType(x, y+1-a, DominoTypeAscender);
+      setDominoState(x, y+1-a, 8);
+      setDominoDir(x, y+1-a, -1);
+      setDominoYOffset(x, y+1-a, 0);
 
-    markDirty(x, y-a);
-    markDirty(x-1, y-a);
-    markDirty(x, y-a+1);
-    markDirty(x-1, y-a+1);
+      markDirty(x, y-a);
+      markDirty(x-1, y-a);
+      markDirty(x, y-a+1);
+      markDirty(x-1, y-a+1);
+    }
+    else
+    {
+      DominoCrash(x, y+1-a, getDominoType(x, y), getDominoExtra(x, y));
+
+      markDirty(x-1, y-1-a);
+      markDirty(x, y-1-a);
+      markDirty(x-1, y-a);
+      markDirty(x, y-a);
+    }
 
     return;
   }
 
   if (getDominoExtra(x, y) != 0x50)
   {
-    setDominoType(x, y-1, getDominoType(x, y));
-    setDominoState(x, y-1, getDominoState(x, y));
-    setDominoDir(x, y-1, getDominoDir(x, y));
-    setDominoYOffset(x, y-1, getDominoYOffset(x, y)+16);
-    setDominoExtra(x, y-1, 0x50);
+    if (getDominoType(x, y-1) == DominoTypeEmpty)
+    {
+      setDominoType(x, y-1, getDominoType(x, y));
+      setDominoState(x, y-1, getDominoState(x, y));
+      setDominoDir(x, y-1, getDominoDir(x, y));
+      setDominoYOffset(x, y-1, getDominoYOffset(x, y)+16);
+      setDominoExtra(x, y-1, 0x50);
+    }
+    else
+    {
+      DominoCrash(x, y-1, getDominoType(x, y), getDominoExtra(x, y));
+
+      markDirty(x-1, y-3);
+      markDirty(x, y-3);
+      markDirty(x-1, y-2);
+      markDirty(x, y-2);
+    }
 
     removeDomino(x, y);
   }
@@ -1236,6 +1329,11 @@ void levelPlayer_c::DTA_O(int x, int y) {
       else
       {
         DominoCrash(x+1, y, getDominoType(x, y), getDominoExtra(x, y));
+
+        markDirty(x, y-2);
+        markDirty(x+1, y-2);
+        markDirty(x, y-1);
+        markDirty(x+1, y-1);
       }
     }
     else
@@ -1253,6 +1351,11 @@ void levelPlayer_c::DTA_O(int x, int y) {
         else
         {
           DominoCrash(x+1, y-1, getDominoType(x, y), getDominoExtra(x, y));
+
+          markDirty(x, y-3);
+          markDirty(x+1, y-3);
+          markDirty(x, y-2);
+          markDirty(x+1, y-2);
         }
       }
     }
@@ -1282,6 +1385,11 @@ void levelPlayer_c::DTA_O(int x, int y) {
     else
     {
       DominoCrash(x, y+1-a, getDominoType(x, y), getDominoExtra(x, y));
+
+      markDirty(x-1, y-1-a);
+      markDirty(x, y-1-a);
+      markDirty(x-1, y-a);
+      markDirty(x, y-a);
     }
 
     markDirty(x, y-a);
@@ -1294,11 +1402,23 @@ void levelPlayer_c::DTA_O(int x, int y) {
 
   if (getDominoExtra(x, y) != 0x50)
   {
-    setDominoType(x, y-1, getDominoType(x, y));
-    setDominoState(x, y-1, getDominoState(x, y));
-    setDominoDir(x, y-1, getDominoDir(x, y));
-    setDominoYOffset(x, y-1, getDominoYOffset(x, y)+16);
-    setDominoExtra(x, y-1, 0x50);
+    if (getDominoType(x, y-1) == DominoTypeEmpty)
+    {
+      setDominoType(x, y-1, getDominoType(x, y));
+      setDominoState(x, y-1, getDominoState(x, y));
+      setDominoDir(x, y-1, getDominoDir(x, y));
+      setDominoYOffset(x, y-1, getDominoYOffset(x, y)+16);
+      setDominoExtra(x, y-1, 0x50);
+    }
+    else
+    {
+      DominoCrash(x, y-1, getDominoType(x, y), getDominoExtra(x, y));
+
+      markDirty(x-1, y-3);
+      markDirty(x, y-3);
+      markDirty(x-1, y-2);
+      markDirty(x, y-2);
+    }
 
     removeDomino(x, y);
   }
@@ -2053,13 +2173,22 @@ int levelPlayer_c::performDominos(ant_c & a) {
 
   a.performAnimation();
 
+  inactive++;
+
   for (int y = 0; y < 13; y++)
     for (int x = 0; x < 20; x++)
       if (getDominoType(x, y) != DominoTypeEmpty &&
           getDominoState(x, y) != 0) {
 
+        int oldState = getDominoState(x, y);
+        int oldExtra = getDominoExtra(x, y);
+        int oldYpos = getDominoYOffset(x, y);
+
         callStateFunction(getDominoType(x, y), getDominoState(x, y), x, y);
 
+        if (oldState != getDominoState(x, y) || oldExtra != getDominoExtra(x, y) || oldYpos != getDominoYOffset(x, y))
+          inactive = 0;
+
         if (getDominoType(x, y) == DominoTypeAscender)
         {
           if (isDirty(x-1, y)) markDirty(x-1, y-1);
@@ -2070,12 +2199,20 @@ int levelPlayer_c::performDominos(ant_c & a) {
 
   timeTick();
 
+  int reason = 0;
+
+  if (finishCheckDone && !levelCompleted(reason))
+  {
+    // we failed after all
+    openExitDoor(false);
+
+    return reason;
+  }
+
   if (triggerIsFalln() && !finishCheckDone)
   {
     finishCheckDone = true;
 
-    int reason = 0;
-
     if (levelCompleted(reason) && !a.carrySomething() && a.isLiving())
     {
       a.success();
@@ -2093,120 +2230,24 @@ int levelPlayer_c::performDominos(ant_c & a) {
     }
   }
 
+  // if level is inactive for a longer time and no ppushed are left
+  if (a.getPushsLeft() == 0 && inactive > 36) {
+    // search for a trigger
+
+    for (int y = 0; y < 13; y++)
+      for (int x = 0; x < 20; x++)
+        if (getDominoType(x, y) == DominoTypeTrigger) {
+
+          // if the trigger is not lying completely flat
+          if (getDominoState(x, y) != 8 && getDominoState(x, y) != 1 && getDominoState(x, y) != 15)
+            return 7;
+
+          x = 20;
+          y = 13;
+        }
+  }
+
   return 0;
 }
 
-bool levelPlayer_c::levelCompleted(int & fail) {
-
-  for (int y = 0; y < 13; y++)
-    for (int x = 0; x < 20; x++) {
-      if (getDominoType(x, y) >= DominoTypeCrash0 &&
-          getDominoType(x, y) <= DominoTypeCrash5)
-      {
-        fail = 3;
-        return false;
-      }
-
-      if (getDominoType(x, y) != DominoTypeEmpty && getDominoType(x, y) != DominoTypeStopper)
-      {
-        if (getDominoType(x, y) == DominoTypeSplitter)
-        { // for splitters is must have started to fall
-          if (getDominoState(x, y) > 2 && getDominoState(x, y) <= 8)
-          {
-            fail = 6;
-            return false;
-          }
-        }
-        else if (getDominoType(x, y) == DominoTypeTrigger)
-        { // triggers must as least have started to fall
-          if (getDominoState(x, y) == 8)
-          {
-            fail = 6;
-            return false;
-          }
-        }
-        else if (getDominoType(x, y) == DominoTypeTumbler)
-        { // tumbler must lie on something or lean against a wall
-          // not falln far enough
-          if (getDominoState(x, y) > 3 && getDominoState(x, y) < 13) {
-            fail = 6;
-            return false;
-          }
-
-          // check if we lean against a step
-          if (getDominoState(x, y) == 3 &&
-              getFg(x-1, y) != FgElementPlatformStep4 &&
-              getFg(x-1, y) != FgElementPlatformStep7)
-          {
-            fail = 6;
-            return false;
-          }
-
-          if (getDominoState(x, y) == 13 &&
-              getFg(x+1, y) != FgElementPlatformStep4 &&
-              getFg(x+1, y) != FgElementPlatformStep7)
-          {
-            fail = 6;
-            return false;
-          }
-
-          // falln far enough but neighbor empty
-          if (   getDominoState(x, y) <= 2
-              && getDominoType(x-1, y) == DominoTypeEmpty
-              && (getDominoType(x-2, y) == DominoTypeEmpty || getDominoState(x-2, y) < 14))
-          {
-            fail = 6;
-            return false;
-          }
-          if (   getDominoState(x, y) >= 14
-              && getDominoType(x+1, y) == DominoTypeEmpty
-              && (getDominoType(x+2, y) == DominoTypeEmpty || getDominoState(x+2, y) > 2)) {
-            fail = 6;
-            return false;
-          }
-
-        }
-        else
-        {
-          if (getDominoState(x, y) == 8)
-          {
-            fail = 4;
-            return false;
-          }
-          if (getDominoState(x, y) > 3 && getDominoState(x, y) < 13)
-          {
-            // here we certainly fail
-            fail = 6;
-            return false;
-          }
-          // in this case we might still succeed, when we lean against a block
-          if (   getDominoState(x, y) == 3
-              && getFg(x-1, y) != FgElementPlatformStep4
-              && (   getDominoType(x-1, y) != DominoTypeStopper
-                  || getDominoType(x+1, y) == DominoTypeEmpty
-                  || (getDominoDir(x+1, y) != -1 && getDominoType(x+1, y) != DominoTypeSplitter))
-             )
-          {
-            // here we lean against a of a blocker and can not go back
-            fail = 6;
-            return false;
-          }
-
-          if (   getDominoState(x, y) == 13
-              && getFg(x+1, y) != FgElementPlatformStep6
-              && (   getDominoType(x+1, y) != DominoTypeStopper
-                  || getDominoType(x-1, y) == DominoTypeEmpty
-                  || (getDominoDir(x-1, y) != 1 && getDominoType(x-1, y) != DominoTypeSplitter))
-             )
-          {
-            // here we lean against a step
-            fail = 6;
-            return false;
-          }
-        }
-      }
-    }
-
-  return true;
-}
 
