@@ -14,77 +14,112 @@
 
 #include <vector>
 #include <fstream>
+#include <stdexcept>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <libintl.h>
 
-void check(int argn, char * argv[], void checker(const ant_c & a, const levelData_c & l)) {
+#include <libosso.h>
+
+static void check_record(const std::string & rec_path, levelsetList_c & levelsetList,
+                         unsigned int & count, unsigned int & failed,
+                         std::string checker(const ant_c & a, const levelData_c & l)) {
   recorder_c rec;
-
-  levelsetList_c levelsetList;
-  levelsetList.load("levels");
+  rec.load(rec_path);
 
   graphicsN_c gr("");
   surface_c surf;
   levelPlayer_c l(surf, gr);
+  levelsetList.getLevelset(rec.getLevelsetName()).loadLevel(l, rec.getLevelName());
+  ant_c a(l, gr, surf);
+
+  while (!rec.endOfRecord()) {
+    a.setKeyStates(rec.getEvent());
+    l.performDoors();
+    l.performDominos(a);
+  }
+
+  // add a few more iterations at the end to make sure the ant has left the level
+  a.setKeyStates(0);
+  for (unsigned int j = 0; j < 100; j++)
+  {
+    l.performDoors();
+    l.performDominos(a);
+  }
+
+  const std::string error = checker(a, l);
+  if (!error.empty()) {
+    std::cout << rec_path << " " << error << std::endl;
+    failed++;
+  }
+  count++;
+}
+
+void check(int argn, char * argv[], std::string checker(const ant_c & a, const levelData_c & l)) {
+  levelsetList_c levelsetList;
+  levelsetList.load("levels");
+
+  unsigned int count = 0;
+  unsigned int failed = 0;
 
   for (int i = 0; i < argn; i++)
   {
-    rec.load(argv[i]);
-
-    levelsetList.getLevelset(rec.getLevelsetName()).loadLevel(l, rec.getLevelName());
-    ant_c a(l, gr, surf);
-
-    while (!rec.endOfRecord()) {
-      a.setKeyStates(rec.getEvent());
-      l.performDoors();
-      l.performDominos(a);
-    }
-
-    // add a few more iterations at the end to make sure the ant has left the level
-    a.setKeyStates(0);
-    for (unsigned int j = 0; j < 100; j++)
+    const std::string path(argv[i]);
+    struct stat st;
+    if (stat(path.c_str(), &st) != 0)
+      throw std::runtime_error("file or directory does not exist: " + path);
+    if (S_ISDIR(st.st_mode))
     {
-      l.performDoors();
-      l.performDominos(a);
+      const std::vector<std::string> entries = directoryEntries(path);
+      for (std::vector<std::string>::const_iterator j = entries.begin(); j != entries.end(); j++) {
+        const std::string & filename = *j;
+        if (filename.size() > 0 && filename[0] != '.')
+          check_record(path + '/' + filename, levelsetList, count, failed, checker);
+      }
+    }
+    else
+    {
+      check_record(path, levelsetList, count, failed, checker);
     }
-
-    std::cout << argv[i];
-    checker(a, l);
-
   }
+
+  std::cout << failed << " out of " << count << " tests failed\n";
 }
 
-void checker1(const ant_c & a, const levelData_c & l) {
+std::string checker1(const ant_c & a, const levelData_c & l) {
+
+    int i;
     // we succeeded, when the ant has vanished, then it went out of the door
-    if (a.isVisible() == false)
-      std::cout << " Level Finished\n";
+    if (a.isVisible() == false && l.levelCompleted(i))
+      return "";
     else
-      std::cout << " Level not Finished\n";
+      return "Level not Finished";
 }
 
-void checker2(const ant_c & a, const levelData_c & l) {
+std::string checker2(const ant_c & a, const levelData_c & l) {
+
+    int i;
     // we succeeded, when the ant has vanished, then it went out of the door
-    if (a.isVisible() == true)
-      std::cout << " Level Failes\n";
+    if (a.isVisible() == true || !l.levelCompleted(i))
+      return "";
     else
-      std::cout << " Level not Failed\n";
+      return "Level not Failed";
 }
 
-void checker3(const ant_c & a, const levelData_c & l) {
+std::string checker3(const ant_c & a, const levelData_c & l) {
   // we succeeded, when the ant has vanished, then it went out of the door
   for (int y = 0; y < 13; y++)
     for (int x = 0; x < 20; x++)
       if (l.getDominoType(x, y) >= levelData_c::DominoTypeCrash0 &&
           l.getDominoType(x, y) <= levelData_c::DominoTypeCrash5) {
-        std::cout << " Crashes happened\n";
-        return;
+        return "";
       }
 
-  std::cout << " Crashes not happened\n";
+  return "Crashes didn't happen";
 }
 
 static std::string getDataDir(void)
@@ -93,6 +128,12 @@ static std::string getDataDir(void)
   return std::string((stat(PKGDATADIR, &st) == 0) ? PKGDATADIR : ".");
 }
 
+static std::string getLocaleDir(void)
+{
+  struct stat st;
+  return std::string((stat(LOCALEDIR, &st) == 0) ? LOCALEDIR : "locale");
+}
+
 static unsigned int getKeyMask(void) {
   unsigned int keyMask = 0;
 
@@ -102,7 +143,7 @@ static unsigned int getKeyMask(void) {
   if ( keystate[SDLK_DOWN] ) keyMask |= KEY_DOWN;
   if ( keystate[SDLK_LEFT] ) keyMask |= KEY_LEFT;
   if ( keystate[SDLK_RIGHT] ) keyMask |= KEY_RIGHT;
-  if ( keystate[SDLK_SPACE] ) keyMask |= KEY_ACTION;
+  if ( keystate[SDLK_LSHIFT] ) keyMask |= KEY_ACTION;
 
   return keyMask;
 }
@@ -115,6 +156,8 @@ static unsigned int getKeyMask(void) {
 // 3 crashes
 // 4 not all dominos fell
 // 5 die
+// 6 trigger not last to fall
+// 7 trigger not flat on the ground
 //
 int playTick(levelPlayer_c & l, ant_c & a)
 {
@@ -125,7 +168,7 @@ int playTick(levelPlayer_c & l, ant_c & a)
   l.drawDominos();
   a.draw();
 
-  if (l.triggerIsFalln() && !a.isVisible() && l.isExitDoorClosed()) {
+  if (l.triggerIsFalln() && !a.isVisible() && l.isExitDoorClosed() && (res == 0)) {
 
     if (l.someTimeLeft())
     {
@@ -167,6 +210,14 @@ typedef enum {
 
 int main(int argc, char * argv[]) {
 
+  osso_context_t *osso_context = NULL;
+  putenv("SDL_VIDEO_X11_WMCLASS=pushover");
+  
+  osso_context = osso_initialize ("pushover", "0.0.2", 0, NULL);
+  if(osso_context == NULL) {
+    fprintf(stderr, "error initiating osso context\n");
+  }
+
   // filter out the no graphic cases, they are special and will be treated
   // separately
   if (argc >= 3 && strcmp(argv[1], "-c") == 0)   // the must complete tests
@@ -187,6 +238,12 @@ int main(int argc, char * argv[]) {
     return 0;
   }
 
+  // setup internationalisation
+  setlocale(LC_MESSAGES, "");
+  bindtextdomain("pushover", getLocaleDir().c_str());
+  bind_textdomain_codeset("pushover", "UTF-8");
+  textdomain("pushover");
+
   // now off to all modes that use graphics
   const std::string datadir = getDataDir();
   std::string selectedMission;  // the mission that was selected in menu
@@ -226,6 +283,7 @@ int main(int argc, char * argv[]) {
       selectedMission = rec.getLevelsetName();
       levelsetList.getLevelset(selectedMission).loadLevel(l, rec.getLevelName());
       a.initForLevel();
+      soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
 
       nextState = ST_REPLAY;
     }
@@ -245,6 +303,7 @@ int main(int argc, char * argv[]) {
 
       levelsetList.getLevelset(selectedMission).loadLevel(l, levelName);
       a.initForLevel();
+      soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
 
       nextState = ST_PREPLAY;
     }
@@ -262,6 +321,7 @@ int main(int argc, char * argv[]) {
       textsections_c sections(file, true);
       l.load(sections);
       selectedMission = "Original";            // TODO we need to find out which levelset this file belongs to
+      soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
 
       nextState = ST_PREPLAY;
     }
@@ -286,6 +346,13 @@ int main(int argc, char * argv[]) {
   unsigned int failReason = 0;
   unsigned int failDelay = 0; // a counter to delay the fail window a bit after failing
 
+  screen.toggleFullscreen();
+  
+  if (nextState == ST_MAIN)
+  {
+    soundSystem_c::instance()->playMusic(datadir+"/themes/option.ogg");
+  }
+
   try {
 
     while (!exitProgram) {
@@ -339,25 +406,13 @@ int main(int argc, char * argv[]) {
             case ST_CONFIG:   window = getConfigWindow(screen, gr); break;
             case ST_SOLVED:   window = getSolvedWindow(screen, gr); break;
             case ST_ABOUT:    window = getAboutWindow(screen, gr); break;
-            case ST_FAILED:
-                              {
-                                std::string reason;
-                                switch (failReason) {
-                                  case 2: reason = "You've been too slow"; break;
-                                  case 3: reason = "Some dominoes crashed"; break;
-                                  case 4: reason = "Not all dominoes fell"; break;
-                                  case 5: reason = "You died"; break;
-                                  case 6: reason = "Trigger was not last to fall"; break;
-                                }
-                                window = getFailedWindow(reason, screen, gr);
-                              }
-                              break;
+            case ST_FAILED:   window = getFailedWindow(failReason, screen, gr); break;
 
             case ST_HELP:
                               {
                                 std::string text;
                                 if (l.someTimeLeft())
-                                  text = "Arrange dominos in a run so that trigger falls last. You have 1 push.";
+                                  text = _("Arrange dominos in a run so that trigger falls last. You have 1 push.");
                                 else
                                   text = l.getHint();
 
@@ -385,6 +440,8 @@ int main(int argc, char * argv[]) {
 
             case ST_EXIT:
                               exitProgram = true;
+			      if (osso_context)
+			        osso_deinitialize(osso_context);
                               break;
 
           }
@@ -436,7 +493,12 @@ int main(int argc, char * argv[]) {
                     screen.markAllDirty();
                     nextState = ST_CONFIGTOG;
                     break;
-                  case 1: break;  // toggle sound effects
+
+                  case 1:  // toggle sound effects
+                    soundSystem_c::instance()->toggleOnOff();
+                    nextState = ST_CONFIGTOG;
+                    break;
+
                   default: nextState = ST_MAIN; break;  // back to main menu
                 }
               }
@@ -484,6 +546,7 @@ int main(int argc, char * argv[]) {
                 {
                   nextState = ST_PREPLAY;
                   ls.loadLevel(l, ls.getLevelNames()[sel]);
+                  soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
                   a.initForLevel();
                 }
               }
@@ -491,21 +554,21 @@ int main(int argc, char * argv[]) {
             break;
 
           case ST_PLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) nextState = ST_QUIT;
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_F1)     nextState = ST_HELP;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q) nextState = ST_QUIT;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_h)     nextState = ST_HELP;
             if (event.type == SDL_KEYDOWN && event.key.keysym.sym == 'r')         rec.save("man");
             break;
 
           case ST_PREPLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) nextState = ST_PLAY;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q) nextState = ST_PLAY;
             break;
 
           case ST_PREREPLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) nextState = ST_REPLAY;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q) nextState = ST_REPLAY;
             break;
 
           case ST_REPLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE)
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q)
             {
               rec.truncate();
               nextState = ST_PLAY;
@@ -550,7 +613,10 @@ int main(int argc, char * argv[]) {
               {
                 switch(dynamic_cast<listWindow_c*>(window)->getSelection())
                 {
-                  case 2: nextState = ST_MAIN; break;    // return to main menu
+                  case 2:
+                    nextState = ST_LEVEL;
+                    soundSystem_c::instance()->playMusic(datadir+"/themes/option.ogg");
+                    break;    // return to level list
                   case 1:
                           {       // restart level
                             nextState = ST_PREPLAY;
@@ -579,8 +645,9 @@ int main(int argc, char * argv[]) {
               {
                 switch(dynamic_cast<listWindow_c*>(window)->getSelection())
                 {
-                  case 0: nextState = ST_LEVEL; break; // select next level to play
-                  case 1: nextState = ST_MAIN; break;  // back to main
+                  case 0:
+                    nextState = ST_LEVEL;
+                    break; // select next level to play
                 }
               }
             }
@@ -623,7 +690,10 @@ int main(int argc, char * argv[]) {
                       else nextState = ST_PREPLAY;
                     }
                     break;
-                  case 1: nextState = ST_MAIN; break;  // back to main
+                  case 1:
+                    nextState = ST_LEVEL;
+                    soundSystem_c::instance()->playMusic(datadir+"/themes/option.ogg");
+                    break;  // back to level list
                 }
               }
             }
@@ -683,21 +753,25 @@ int main(int argc, char * argv[]) {
             a.setKeyStates(keyMask);
           }
           failReason = playTick(l, a);
-          switch (failReason) {
-            case 1:
-              rec.save("sol");
-              solved.addLevel(l.getChecksum());
-              nextState = ST_SOLVED;
-              break;
-            case 0:
-              break;
-            case 2:
-              nextState = ST_FAILED;
-              break;
-            default:
-              failDelay = 36;
-              nextState = ST_FAILDELAY;
-              break;
+
+          if (l.levelInactive())
+          {
+            switch (failReason) {
+              case 1:
+                rec.save("sol");
+                solved.addLevel(l.getChecksum());
+                nextState = ST_SOLVED;
+                break;
+              case 0:
+                break;
+              case 2:
+                nextState = ST_FAILED;
+                break;
+              default:
+                failDelay = 36;
+                nextState = ST_FAILDELAY;
+                break;
+            }
           }
           break;
 
