@@ -46,6 +46,11 @@ class soundSystem_c {
      */
     static soundSystem_c * instance(void);
 
+    /* toggle quiet sound effects */
+    void toggleOnOff(void) { quiet = !quiet; }
+
+    void playMusic(const std::string & fname);
+
   private:
 
     struct soundDat {
@@ -64,10 +69,14 @@ class soundSystem_c {
      */
     bool useSound;
 
+    bool quiet;  // used to toggle sound effects indepentently of sound
+
     std::vector<struct soundDat> sounds;
 
     static class soundSystem_c *inst;
 
+    Mix_Music * music;
+
 };
 
 #endif
