@@ -21,7 +21,7 @@ std::string getHome(void) {
 
 #ifdef WIN32
 
-  static char userHome[MAX_PATH];
+  static char userHome[MAX_PATH+1];
 
   HKEY key;
   DWORD size = MAX_PATH;
@@ -38,12 +38,10 @@ std::string getHome(void) {
     RegCloseKey(key);
 
   size = strlen(userHome);
-  userHome[size] = '\\';
-  userHome[size+1] = '\0';
+  userHome[size] = '\0';
 
   std::string home = std::string(userHome) + "\\pushover\\";
 
-
 #else
 
   std::string home = std::string(getenv("HOME"))+"/.pushover/";
@@ -60,15 +58,20 @@ std::string getHome(void) {
 #else
     if (::mkdir(home.c_str(), S_IRWXU) != 0)
 #endif
-      throw std::runtime_error("Can not create home directory\n");
-  }
-  else
-  {
-    if (::closedir(dir) != 0)
-      throw std::runtime_error("Can not close dir in find home\n");
+      throw std::runtime_error("Can't create home directory: " + home);
   }
 
   return home;
 }
 
-
+std::vector<std::string> directoryEntries(const std::string & path) {
+  DIR * dir = ::opendir(path.c_str());
+  if (dir == NULL)
+    throw std::runtime_error("Can't open directory: " + path);
+  std::vector<std::string> entries;
+  for (struct dirent * i = ::readdir(dir); i != NULL; i = ::readdir(dir))
+    entries.push_back(i->d_name);
+  if (::closedir(dir) != 0)
+    throw std::runtime_error("Can't close directory: " + path);
+  return entries;
+}
